@@ -1,20 +1,17 @@
-
-
 EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,iter.max,precision)
 {
   start.time <- Sys.time()
-  pb = tkProgressBar(title = "DEC-tLMEC by EM", min = 0,max = iter.max, width = 300)
+  pb = tkProgressBar(title = "DEC-T-tLMEC by EM", min = 0,max = iter.max, width = 300)
   setTkProgressBar(pb, 0, label=paste("Iter ",0,"/",iter.max,"     -     ",0,"% done",sep = ""))
   
-  
-  GB = GenzBretz(maxpts = 5e4, abseps = 1e-9, releps = 0)
+
   if(cens.type=="left"){
     LL=rep(-Inf,length(cc))
     LU=rep(Inf,length(cc))
     LU[cc==1]=y[cc==1]
     LL=as.vector(LL)
     LU=as.vector(LU)
-  }
+  } 
   
   if(cens.type=="right"){
     LL=rep(-Inf,length(cc))
@@ -44,29 +41,58 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
     iD1 <- solve(D1)
     iD1 <- (iD1 + t(iD1))/2
     nu <- initial$nu
-    phis<-initial$phi
     
-    if(struc=="DEC")
-    {
-      phi1 <- initial$phi1
-      phi2 <- initial$phi2
+    if(!is.null(initial$phi)){
+      phis<-initial$phi
       
-      teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,phi2,nu)
+      if(struc=="DEC")
+      {
+        phi1 <- initial$phi1
+        phi2 <- initial$phi2
+        
+        teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,phi2,nu)
+        
+      } else if(struc=="DEC(AR)"){
+        phi1 <- initial$phi1
+        phi2 <- 1
+        teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,nu)
+        
+      } else if(struc=="SYM"){
+        phi1 <- initial$phi1
+        phi2 <- 0
+        teta <- c(beta1, sigmae,D1[upper.tri(D1, diag = T)],phi1,nu)
+        
+      } else {
+        phi1 <- NULL
+        phi2 <- NULL
+        teta <- c(beta1, sigmae,D1[upper.tri(D1, diag = T)],nu)
+      }
+    }
+    if(is.null(initial$phi)){
       
-    } else if(struc=="DEC(AR)"){
-      phi1 <- initial$phi1
-      phi2 <- 1
-      teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,nu)
       
-    } else if(struc=="SYM"){
-      phi1 <- initial$phi1
-      phi2 <- 0
-      teta <- c(beta1, sigmae,D1[upper.tri(D1, diag = T)],phi1,nu)
-      
-    } else {
-      phi1 <- NULL
-      phi2 <- NULL
-      teta <- c(beta1, sigmae,D1[upper.tri(D1, diag = T)],nu)
+      if(struc=="DEC")
+      {
+        phi1 <- 0.1
+        phi2 <- 1
+        
+        teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,phi2,nu)
+        
+      } else if(struc=="DEC(AR)"){
+        phi1 <- 0.1
+        phi2 <- 1
+        teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,nu)
+        
+      } else if(struc=="SYM"){
+        phi1 <- 0.1
+        phi2 <- 0
+        teta <- c(beta1, sigmae,D1[upper.tri(D1, diag = T)],phi1,nu)
+        
+      } else {
+        phi1 <- NULL
+        phi2 <- NULL
+        teta <- c(beta1, sigmae,D1[upper.tri(D1, diag = T)],nu)
+      }
     }
     
   }
@@ -76,11 +102,11 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
     sigmae= 0.25 
     D1=0.1*diag(dim(z)[2])
     nu=3
-     iD1<- solve(D1)
+    iD1<- solve(D1)
     if(struc=="DEC")
     {
       phi1 <- 0.1
-      phi2 <- 0.1
+      phi2 <- 1
       
       teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,phi2,nu)
       
@@ -107,7 +133,7 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
   
   criterio <- 1
   count <- 0
-
+  
   
   loglik <- logliktslmec(nu=nu,y=y,x=x,z=z,cc=cc,ttc=ttc,nj=nj,LL=LL,LU=LU,betas=beta1,sigmae=sigmae,D1=D1,phi1=phi1,phi2=phi2,struc=struc)
   
@@ -117,7 +143,7 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
   while(criterio > precision){
     
     count <- count + 1
-  
+    
     
     soma1 <- matrix(0,q1,q1)
     soma2 <- 0
@@ -138,9 +164,9 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
     biest <- matrix(0,m2,m) 
     yhi <- matrix(0,N,1)   
     
-   
+    
     for (j in 1:m){
-   
+      
       cc1 <- cc[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
       y1 <- y[(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]
       x1 <- matrix(x[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),  ],ncol=p)
@@ -175,7 +201,7 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
         
         yh <- matrix(y1,nj[j],1)
         best <- (Lambda1%*%(t(z1)*(1/sigmae))%*%invGama)%*%(yh - muii)
-    
+        
         Eu2yy <- (cdm^2)*(y1%*%t(y1))
         Eu2y <- (cdm^2)*(matrix(y1,nj[j],1))
         Eu2 <- cdm^2 
@@ -183,16 +209,19 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
         E2 <- Eu2yy - Eu2y%*%t(muii) - muii%*%t(Eu2y) + Eu2*(muii)%*%t(muii)
         E1 <- (uy - u*(muii))%*%t(uy - u*(muii))
         
-       
+        
       }
       if(sum(cc1)>=1)
       {
         if(sum(cc1)==nj[j])
         {
+                
           
-             aux1U <- MomTrunc::pmvnormt(lower = as.vector(LL1),upper = as.vector(LU1),mean = as.vector(muii),sigma = as.matrix((nu/(nu + 2))*SIGMA),nu = (nu+2))
-          aux2U <- MomTrunc::pmvnormt(lower = as.vector(LL1),upper=as.vector(LU1), mean = as.vector(muii),sigma = as.matrix(SIGMA),nu=nu)
           
+          aux1U <- TruncatedNormal::pmvt(lb = as.vector(LL1), ub = as.vector(LU1), mu = as.vector(muii),  df=(nu+2), sigma = as.matrix((nu/(nu + 2))*SIGMA))
+          aux2U <- TruncatedNormal::pmvt(lb = as.vector(LL1), ub = as.vector(LU1), mu = as.vector(muii),  df=(nu+2), sigma = as.matrix(SIGMA))
+          
+               
           u <- as.numeric(aux1U/aux2U)
           
           auxy <-relliptical::mvtelliptical(lower = as.vector(LL1),upper=as.vector(LU1),mu = as.vector(muii), Sigma = as.matrix((nu/(nu + 2))*SIGMA),dist = "t",nu=(nu+2))
@@ -208,21 +237,21 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
           auxb <- relliptical::mvtelliptical(lower = as.vector(LL1),upper=as.vector(LU1),mu = as.vector(muii), Sigma = as.matrix(SIGMA),dist = "t",nu=(nu))
           yh <- auxb$EY
           best <- (Lambda1%*%(t(z1)*(1/sigmae))%*%invGama)%*%(yh - muii)
-           
+          
           cp <- (((nu+nj[j])/nu)^2)*((gamma((nu+nj[j])/2)*gamma((nu+4)/2))/(gamma(nu/2)*gamma((nu+nj[j]+4)/2)))
           
           auxw <- relliptical::mvtelliptical(lower = as.vector(LL1),upper=as.vector(LU1),mu = as.vector(muii), Sigma = as.matrix((nu/(nu + 4))*SIGMA),dist = "t",nu=(nu+4))
           
-           auxEU <- MomTrunc::pmvnormt(lower = as.vector(LL1),upper=as.vector(LU1), mean = as.vector(muii),sigma = as.matrix((nu/(nu + 4))*SIGMA),nu=(nu+4))
+           
+          auxEU <-TruncatedNormal::pmvt(lb = as.vector(LL1), ub = as.vector(LU1), mu = as.vector(muii),  df=(nu+4), sigma = as.matrix((nu/(nu + 4))*SIGMA))
           
-          
-          Eu2yy <- cp*(auxEU/aux2U)*auxw$EYY
+         Eu2yy <- cp*(auxEU/aux2U)*auxw$EYY
           Eu2y <- cp*(auxEU/aux2U)*auxw$EY
           Eu2 <- cp*(auxEU/aux2U)
           
           E2 <- Eu2yy - Eu2y%*%t(muii) - muii%*%t(Eu2y) + Eu2*(muii)%*%t(muii)
           E1 <- (uy - u*(muii))%*%t(uy - u*(muii))
-            }
+        }
         else{
           
           muiic <-  W1[cc1==1,]%*%gamma1 + SIGMA[cc1==1,cc1==0]%*%solve(SIGMA[cc1==0,cc1==0])%*%(y1[cc1==0]-W1[cc1==0,]%*%gamma1)
@@ -239,10 +268,12 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
           
           LL1c <- LL1[cc1==1]
           LU1c <- LU1[cc1==1]
-  
-          aux1U <- MomTrunc::pmvnormt(lower = as.vector(LL1c),upper = as.vector(LU1c),mean = as.vector(muiic),sigma = as.matrix(Sc0til),nu = (nu + 2 + length(cc1[cc1==0])))
-          aux2U <- MomTrunc::pmvnormt(lower = as.vector(LL1c),upper=as.vector(LU1c), mean = as.vector(muiic),sigma =as.matrix(Sc0),nu= (nu + length(cc1[cc1==0])))
+           
           
+          aux1U <- TruncatedNormal::pmvt(lb = as.vector(LL1c), ub = as.vector(LU1c), mu = as.vector(muiic),  df=(nu + 2 + length(cc1[cc1==0])), sigma = as.matrix(Sc0til))
+          aux2U <- TruncatedNormal::pmvt(lb = as.vector(LL1c), ub = as.vector(LU1c), mu = as.vector(muiic),  df=(nu + length(cc1[cc1==0])), sigma = as.matrix(Sc0))
+          
+                
           u <- as.numeric(aux1U/aux2U)*(1/auxQy0)
           Sc0til=round((Sc0til+t(Sc0til))/2,3)
           
@@ -269,15 +300,16 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
           
           best <- (Lambda1%*%(t(z1)*(1/sigmae))%*%invGama)%*%(yh - muii)
           
-        
+          
           
           dp <- ((nu+nj[j])^2)*((gamma((nu+nj[j])/2)*gamma((nu+4+length(cc1[cc1==0]))/2))/(gamma((nu+length(cc1[cc1==0]))/2)*gamma((nu+4+nj[j])/2)))
           
           Sirc=(nu + Qy0)/(nu + 4 + length(cc1[cc1==0]))*Si
           Sirc=round(((t(Sirc)+Sirc)/2),2)
-          auxEU <- MomTrunc::pmvnormt(lower = as.vector(LL1c),upper=as.vector(LU1c), mean = as.vector(muiic),sigma = as.matrix(Sirc),nu=(nu + 4 + length(cc1[cc1==0])))
           
-          auxEw <- relliptical::mvtelliptical(lower = as.vector(LL1c),upper=as.vector(LU1c),mu = as.vector(muiic), Sigma = as.matrix(Sirc),dist = "t",nu=(nu + 4 + length(cc1[cc1==0])))
+          auxEU <-TruncatedNormal::pmvt(lb = as.vector(LL1c), ub = as.vector(LU1c), mu = as.vector(muiic),  df=(nu + 4 + length(cc1[cc1==0])), sigma = as.matrix(Sirc))
+          
+        auxEw <- relliptical::mvtelliptical(lower = as.vector(LL1c),upper=as.vector(LU1c),mu = as.vector(muiic), Sigma = as.matrix(Sirc),dist = "t",nu=(nu + 4 + length(cc1[cc1==0])))
           
           Ew1aux <- auxEw$EY
           Ew2aux <- auxEw$EYY
@@ -295,7 +327,7 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
           E2 <- Eu2yy - Eu2y%*%t(muii) - muii%*%t(Eu2y) + Eu2*(muii)%*%t(muii)
           E1 <- (uy - u*(muii))%*%t(uy - u*(muii))
           
-               
+          
         }    
         
       } 
@@ -305,10 +337,10 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
                         + t(muii)%*%invGama%*%z1%*%ub  + t(ub)%*%t(z1)%*%invGama%*%muii + u*t(muii)%*%invGama%*%muii + sum(diag(ubb%*%t(z1)%*%invGama%*%z1))) 
       soma3 <- soma3 + (u*t(x1)%*%invGama%*%x1)
       soma4 <- soma4 + (t(x1)%*%invGama%*%(uy - z1%*%ub))
-         
-   
+      
+      
       soma7 <- soma7 +  (((nu+nj[j])/(nu+nj[j]+2))*t(x1)%*%SIGMAinv%*%x1 - ((nu+nj[j]+2)/(nu+nj[j]))*t(x1)%*%SIGMAinv%*%(E2)%*%SIGMAinv%*%x1 + (t(x1)%*%SIGMAinv%*%(E1)%*%SIGMAinv%*%x1))
-        
+      
       ui[j] <- u
       uyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),j] <- uy
       uyyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]<- uyy
@@ -325,7 +357,7 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
     yfit <- apply(yest,1,sum)  
     yfit[cc==1] <- yhatorg[cc==1]
     
-   
+    
     Infbetas[1:p,1:p] <- soma7
     
     Infbetas <- (Infbetas + t(Infbetas))/2         
@@ -337,28 +369,28 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
     D1 <- (1/m)*(soma1)
     iD1 <- solve(D1)
     
-   
+    
     if(nu.fixed==FALSE)
     {
       nu <- optimize(f = logliktslmec,interval = c(3,30),tol = 0.00001, maximum = TRUE,y=y,x=x,z=z,cc=cc,ttc=ttc,nj=nj,
                      LL=LL,LU=LU,betas=beta1,sigmae=sigmae,D1=D1,phi1=phi1,phi2=phi2,struc=struc)$maximum
     }
     
-  
+    
     if(struc=="DEC")                                                                     
     {
       phis <- optim(c(phi1,phi2), FCit,lower =c(0.01,0.01), upper=c(0.9,30),method = "L-BFGS-B", hessian=TRUE, beta1=beta1,sigmae=sigmae,ttc=ttc,ubi=ubi,ubbi=ubbi,uybi=uybi,uyyi=uyyi,uyi=uyi,ui=ui,x=x,z=z,nj=nj,struc=struc)$par
       phi1 <- phis[1]
       phi2 <- phis[2]
-     teta1 <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,phi2,nu)
-     
+      teta1 <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],phi1,phi2,nu)
+      
     } else if(struc=="DEC(AR)"){
       phi2 <- 1                                   
       phi1 <- optimize(f=FCiphi1t, lower= 0.0001, upper=0.9, tol = 0.001, phi2=phi2, beta1=beta1,sigmae=sigmae,
                        ttc=ttc,ubi=ubi,ubbi=ubbi,uybi=uybi,uyyi=uyyi,uyi=uyi,ui=ui,x=x,z=z,nj=nj,struc=struc)$minimum
-     
+      
       teta1 <- c(beta1, sigmae,D1[upper.tri(D1, diag = T)],phi1,nu)
-    
+      
     } else if(struc=="SYM"){
       phi2 <- 0
       phi1 <- optimize(f=FCiphi1t, lower= 0.0001, upper=0.9,tol = 0.001, phi2=phi2, beta1=beta1,sigmae=sigmae,
@@ -367,26 +399,26 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
       teta1 <- c(beta1, sigmae,D1[upper.tri(D1, diag = T)],phi1,nu)
       
     } else{
-     
+      
       teta1 <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],nu)
-  
+      
     }
     
     
     
     loglik <- logliktslmec(nu=nu,y=y,x=x,z=z,cc=cc,ttc=ttc,nj=nj,LL=LL,LU=LU,betas=beta1,sigmae=sigmae,D1=D1,phi1=phi1,phi2=phi2,struc=struc)
-       loglikp1 <- loglik 
-       
-       
-       
-       if(count > 1){
-         criterio <- sqrt(((loglikp1/loglikp)-1)%*%((loglikp1/loglikp)-1))
-         setTkProgressBar(pb, count, label=paste("Iter ",count,"/",iter.max,"     -     ",floor((count)/(iter.max)*100),"% done",sep = ""))
-       }    
-       if(count==iter.max){criterio <- precision*0.0001}
-
-     
-       
+    loglikp1 <- loglik 
+    
+    
+    
+    if(count > 1){
+      criterio <- sqrt(((loglikp1/loglikp)-1)%*%((loglikp1/loglikp)-1))
+      setTkProgressBar(pb, count, label=paste("Iter ",count,"/",iter.max,"     -     ",floor((count)/(iter.max)*100),"% done",sep = ""))
+    }    
+    if(count==iter.max){criterio <- precision*0.0001}
+    
+    
+    
     teta <- teta1
     loglikp <- loglikp1
     
@@ -444,7 +476,7 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
   }
   if(struc=="UNC"){ phi=NULL; tableP =NULL }
   
-   
+  
   nnp=0
   for(al in 1:dim(D1)[1]) 
   {noa=paste(1:al,al,sep = "")
@@ -464,7 +496,7 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
                   AIC=AICc, BIC=BICc, iter = count,
                   ubi = ubi, ubbi = ubbi, uybi = uybi, uyi = uyi, uyyi = uyyi,ui=ui, MI=Infbetas,residual=res,
                   Prev= NULL, time=time.taken, SE=SE,tableB=tableB,tableS=tableS,tableP=tableP,
-                  tableA=tableA,yest=yfit, yog = yhatorg)
+                  tableA=tableA,yfit=yfit, yorg = yhatorg)
   
   if  (count == iter.max)
   {
@@ -478,7 +510,7 @@ EMCensDECT<- function(cc,y,x,z,ttc,nj,struc,initial,cens.type,LL,LU,nu.fixed,ite
   }
   
   class(obj.out) <- "DECTLMEC"
- 
+  
   return(obj.out)
 }
 
@@ -486,11 +518,11 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
 {
   
   start.time <- Sys.time()
-  pb = tkProgressBar(title = "AR(p)-tLMEC by EM", min = 0,max = iter.max, width = 300)
+  pb = tkProgressBar(title = "AR(p)-T-LMEC by EM", min = 0,max = iter.max, width = 300)
   setTkProgressBar(pb, 0, label=paste("Iter ",0,"/",iter.max,"     -     ",0,"% done",sep = ""))
   
   
-  GB = GenzBretz(maxpts = 5e4, abseps = 1e-9, releps = 0)
+  
   if(cens.type=="left"){
     LL=rep(-Inf,length(cc))
     LU=rep(Inf,length(cc))
@@ -514,14 +546,14 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
     LU=as.vector(LU)
   }
   
-
+  
   m <- length(nj)[1]
   N <- sum(nj)
   q1 <- dim(z)[2]
   m2 <- m*q1
   p <- dim(x)[2]
   
-
+  
   if(!is.null(initial)){
     beta1 <- matrix(initial$betas,p,1)
     sigmae <- initial$sigma2
@@ -529,16 +561,27 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
     iD1 <- solve(D1)
     iD1 <- (iD1 + t(iD1))/2
     nu <- initial$nu
-    pii = as.numeric(pacf((y - x%*%beta1),lag.max=Arp,plot=F)$acf) 
-    phis<-initial$phi
+    if(!is.null(initial$phi)){
+      pii = as.numeric(pacf((y - x%*%beta1),lag.max=Arp,plot=F)$acf) 
+      phis<-initial$phi
+      if(Arp!="UNC"){
+        pii = as.numeric(pacf((y - x%*%beta1),lag.max=Arp,plot=F)$acf)
+        phi = phis}
+      if(Arp=="UNC"){
+        pii=0
+        Arp=0
+        phi = 0}
+    }
+    if(is.null(initial$phi)){
+      if(Arp!="UNC"){
+        pii = as.numeric(pacf((y - x%*%beta1),lag.max=Arp,plot=F)$acf)
+        phi =  estphit(pii)}
+      if(Arp=="UNC"){
+        pii=0
+        Arp=0
+        phi = 0}
+    }
     
-    if(Arp!="UNC"){
-      pii = as.numeric(pacf((y - x%*%beta1),lag.max=Arp,plot=F)$acf)
-      phi = phis}
-    if(Arp=="UNC"){
-      pii=0
-      Arp=0
-      phi = 0}
   }
   
   if(is.null(initial)){ 
@@ -556,16 +599,16 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
       pii=0
       phi = 0}
   }
-
-    qr <- length(D1[lower.tri(D1, diag = T)])
+  
+  qr <- length(D1[lower.tri(D1, diag = T)])
   W <- x
   gamma1 <- as.vector(c(beta1))
   
-
+  
   
   teta <- c(beta1,sigmae,D1[upper.tri(D1, diag = T)],pii,nu)
-
-
+  
+  
   criterio <- 1
   count <- 0
   
@@ -581,7 +624,7 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
     soma2 <- 0
     soma3 <- matrix(0,p,p)
     soma4 <- matrix(0,p,1)
-     soma7 <- matrix(0,p,p)
+    soma7 <- matrix(0,p,p)
     Infbetas <- matrix(0,p,p)
     
     ui <- rep(0,m) 
@@ -617,8 +660,8 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
         eGamma<-MatArp(pii,tt1,sigmae)
         Gama <-eGamma/sigmae
       }
-        
-     
+      
+      
       
       invGama <- solve(Gama)
       SIGMA <- (sigmae*Gama + (z1)%*%D1%*%t(z1)) 
@@ -643,7 +686,7 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
         yh <- matrix(y1,nj[j],1)
         best <- (Lambda1%*%(t(z1)*(1/sigmae))%*%invGama)%*%(yh - muii)
         
-       
+        
         
         Eu2yy <- (cdm^2)*(y1%*%t(y1))
         Eu2y <- (cdm^2)*(matrix(y1,nj[j],1))
@@ -652,18 +695,20 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
         E2 <- Eu2yy - Eu2y%*%t(muii) - muii%*%t(Eu2y) + Eu2*(muii)%*%t(muii)
         E1 <- (uy - u*(muii))%*%t(uy - u*(muii))
         
-            }
+      }
       if(sum(cc1)>=1)
       {
         if(sum(cc1)==nj[j])
         {
-          aux1U <- MomTrunc::pmvnormt(lower = as.vector(LL1),upper = as.vector(LU1),mean = as.vector(muii),sigma = as.matrix((nu/(nu + 2))*SIGMA),nu = (nu+2))
-          aux2U <- MomTrunc::pmvnormt(lower = as.vector(LL1),upper=as.vector(LU1), mean = as.vector(muii),sigma = as.matrix(SIGMA),nu=nu)
           
+          aux1U <- TruncatedNormal::pmvt(lb = as.vector(LL1), ub = as.vector(LU1), mu = as.vector(muii),  df=(nu + 2), sigma = as.matrix((nu/(nu + 2))*SIGMA))
+          aux2U <- TruncatedNormal::pmvt(lb = as.vector(LL1), ub = as.vector(LU1), mu = as.vector(muii),  df=nu, sigma = as.matrix(SIGMA))
+          
+              
           u <- as.numeric(aux1U/aux2U)
           
           auxy <- relliptical::mvtelliptical(lower = as.vector(LL1),upper=as.vector(LU1),mu = as.vector(muii), Sigma = as.matrix((nu/(nu + 2))*SIGMA),dist = "t",nu=(nu+2))
-            
+          
           uy <- u*auxy$EY
           uyy <- u*auxy$EYY
           
@@ -676,13 +721,16 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
           yh <- auxb$EY
           best <- (Lambda1%*%(t(z1)*(1/sigmae))%*%invGama)%*%(yh - muii)
           
-      
+          
           
           cp <- (((nu+nj[j])/nu)^2)*((gamma((nu+nj[j])/2)*gamma((nu+4)/2))/(gamma(nu/2)*gamma((nu+nj[j]+4)/2)))
           
           auxw <- relliptical::mvtelliptical(lower = as.vector(LL1),upper=as.vector(LU1),mu = as.vector(muii), Sigma = as.matrix((nu/(nu + 4))*SIGMA),dist = "t",nu=(nu+4))
           
-         auxEU <- MomTrunc::pmvnormt(lower = as.vector(LL1),upper=as.vector(LU1), mean = as.vector(muii),sigma = as.matrix((nu/(nu + 4))*SIGMA),nu=(nu+4))
+         
+          
+          auxEU <- TruncatedNormal::pmvt(lb = as.vector(LL1), ub = as.vector(LU1), mu = as.vector(muii),  df=(nu +4), sigma = as.matrix((nu/(nu + 4))*SIGMA))
+              
           
           
           Eu2yy <- cp*(auxEU/aux2U)*auxw$EYY
@@ -692,7 +740,7 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
           E2 <- Eu2yy - Eu2y%*%t(muii) - muii%*%t(Eu2y) + Eu2*(muii)%*%t(muii)
           E1 <- (uy - u*(muii))%*%t(uy - u*(muii))
           
-            }
+        }
         else{
           
           muiic <-  W1[cc1==1,]%*%gamma1 + SIGMA[cc1==1,cc1==0]%*%solve(SIGMA[cc1==0,cc1==0])%*%(y1[cc1==0]-W1[cc1==0,]%*%gamma1)
@@ -709,9 +757,10 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
           
           LL1c <- LL1[cc1==1]
           LU1c <- LU1[cc1==1]
-         
-          aux1U <- MomTrunc::pmvnormt(lower = as.vector(LL1c),upper = as.vector(LU1c),mean = as.vector(muiic),sigma = as.matrix(Sc0til),nu = (nu + 2 + length(cc1[cc1==0])))
-          aux2U <- MomTrunc::pmvnormt(lower = as.vector(LL1c),upper=as.vector(LU1c), mean = as.vector(muiic),sigma =as.matrix(Sc0),nu= (nu + length(cc1[cc1==0])))
+        
+          aux1U <- TruncatedNormal::pmvt(lb = as.vector(LL1c), ub = as.vector(LU1c), mu = as.vector(muiic),  df=(nu + 2 + length(cc1[cc1==0])), sigma = as.matrix(Sc0til))
+          aux2U <- TruncatedNormal::pmvt(lb = as.vector(LL1c), ub = as.vector(LU1c), mu = as.vector(muiic),  df=(nu + length(cc1[cc1==0])), sigma = as.matrix(Sc0))
+          
           
           u <- as.numeric(aux1U/aux2U)*(1/auxQy0)
           Sc0til=round((Sc0til+t(Sc0til))/2,3)
@@ -738,13 +787,15 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
           
           best <- (Lambda1%*%(t(z1)*(1/sigmae))%*%invGama)%*%(yh - muii)
           
-           dp <- ((nu+nj[j])^2)*((gamma((nu+nj[j])/2)*gamma((nu+4+length(cc1[cc1==0]))/2))/(gamma((nu+length(cc1[cc1==0]))/2)*gamma((nu+4+nj[j])/2)))
+          dp <- ((nu+nj[j])^2)*((gamma((nu+nj[j])/2)*gamma((nu+4+length(cc1[cc1==0]))/2))/(gamma((nu+length(cc1[cc1==0]))/2)*gamma((nu+4+nj[j])/2)))
           
           Sirc=as.numeric((nu + Qy0)/(nu + 4 + length(cc1[cc1==0])))*Si
           Sirc=round(Sirc,3)
-          auxEU <- MomTrunc::pmvnormt(lower = as.vector(LL1c),upper=as.vector(LU1c), mean = as.vector(muiic),sigma = as.matrix(Sirc),nu=(nu + 4 + length(cc1[cc1==0])))
           
-          auxEw <- relliptical::mvtelliptical(lower = as.vector(LL1c),upper=as.vector(LU1c),mu = as.vector(muiic), Sigma = as.matrix(Sirc),dist = "t",nu=(nu + 4 + length(cc1[cc1==0])))
+        
+          auxEU <- TruncatedNormal::pmvt(lb = as.vector(LL1c), ub = as.vector(LU1c), mu = as.vector(muiic),  df=(nu + 2 + length(cc1[cc1==0])), sigma = as.matrix(Sc0til))
+          
+         auxEw <- relliptical::mvtelliptical(lower = as.vector(LL1c),upper=as.vector(LU1c),mu = as.vector(muiic), Sigma = as.matrix(Sirc),dist = "t",nu=(nu + 4 + length(cc1[cc1==0])))
           
           Ew1aux <- auxEw$EY
           Ew2aux <- auxEw$EYY
@@ -761,7 +812,7 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
           
           E2 <- Eu2yy - Eu2y%*%t(muii) - muii%*%t(Eu2y) + Eu2*(muii)%*%t(muii)
           E1 <- (uy - u*(muii))%*%t(uy - u*(muii))
-           }       
+        }       
         
       } 
       
@@ -770,7 +821,7 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
                         + t(muii)%*%invGama%*%z1%*%ub  + t(ub)%*%t(z1)%*%invGama%*%muii + u*t(muii)%*%invGama%*%muii + sum(diag(ubb%*%t(z1)%*%invGama%*%z1))) 
       soma3 <- soma3 + (u*t(x1)%*%invGama%*%x1)
       soma4 <- soma4 + (t(x1)%*%invGama%*%(uy - z1%*%ub))
-     soma7 <- soma7 +  (((nu+nj[j])/(nu+nj[j]+2))*t(x1)%*%SIGMAinv%*%x1 - ((nu+nj[j]+2)/(nu+nj[j]))*t(x1)%*%SIGMAinv%*%(E2)%*%SIGMAinv%*%x1 + (t(x1)%*%SIGMAinv%*%(E1)%*%SIGMAinv%*%x1))
+      soma7 <- soma7 +  (((nu+nj[j])/(nu+nj[j]+2))*t(x1)%*%SIGMAinv%*%x1 - ((nu+nj[j]+2)/(nu+nj[j]))*t(x1)%*%SIGMAinv%*%(E2)%*%SIGMAinv%*%x1 + (t(x1)%*%SIGMAinv%*%(E1)%*%SIGMAinv%*%x1))
       ui[j] <- u
       uyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),j] <- uy
       uyyi[(sum(nj[1:j-1])+1) : (sum(nj[1:j])),(sum(nj[1:j-1])+1) : (sum(nj[1:j]))]<- uyy
@@ -789,8 +840,8 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
     yfit <- apply(yest,1,sum)   
     yfit[cc==1] <- yhatorg[cc==1]
     
-      Infbetas[1:p,1:p] <- soma7
-      
+    Infbetas[1:p,1:p] <- soma7
+    
     Infbetas <- (Infbetas + t(Infbetas))/2         
     beta1 <- solve(soma3)%*%soma4
     gamma1 <- as.vector(c(beta1))
@@ -799,37 +850,37 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
     D1 <- (1/m)*(soma1)
     iD1 <- solve(D1)
     
-     
+    
     if(nu.fixed==FALSE)
     {
       nu <- optimize(f = logliktArplmec, interval = c(3,50),tol = 0.00001, maximum = TRUE,y=y,x=x,z=z,cc=cc,ttc=ttc,nj=nj,LL=LL,LU=LU,betas=beta1,sigmae=sigmae,D1=D1,pii=pii)$maximum
-     
+      
     }
     
     
     if(Arp!=0){
       pii <- optim(pii, method = "L-BFGS-B", FCiArpt, lower =rep(-.999,Arp), upper =rep(.999,Arp), beta1=beta1,sigmae=sigmae,ttc=ttc,ubi=ubi,ubbi=ubbi,uybi=uybi,uyyi=uyyi,uyi=uyi,ui=ui,x=x,z=z,nj=nj,hessian=TRUE)$par
       phi=estphit(pii)
-      }
+    }
     if(Arp==0){phi=0}
     
     
-     
-     
- 
+    
+    
+    
     
     loglik <- logliktArplmec(nu=nu,y=y,x=x,z=z,cc=cc,ttc=ttc,nj=nj,LL=LL,LU=LU,betas=beta1,sigmae=sigmae,D1=D1,pii)
-      loglikp1 <- loglik
+    loglikp1 <- loglik
     
     
     if(count > 1){
       criterio <- sqrt(((loglikp1/loglikp)-1)%*%((loglikp1/loglikp)-1))
       setTkProgressBar(pb, count, label=paste("Iter ",count,"/",iter.max,"     -     ",floor((count)/(iter.max)*100),"% done",sep = ""))
-          }    
-      if(count==iter.max){criterio <- precision*0.0001}
+    }    
+    if(count==iter.max){criterio <- precision*0.0001}
     
     loglikp <- loglikp1
-
+    
   } 
   for (k in 1:length(nj)) 
   {tc<-ttc[(sum(nj[1:k-1])+1) : (sum(nj[1:k]))]
@@ -841,7 +892,7 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
                                                                                                                                     -x[(sum(nj[1:k-1])+1) : (sum(nj[1:k])),]%*%beta1)) 
   }
   dd <- D1[upper.tri(D1, diag = T)]
- 
+  
   npar <- length(c(teta))
   AICc <- -2*loglikp + 2*npar
   BICc <- -2*loglikp + log(N)*npar
@@ -855,13 +906,13 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
   
   
   tableS  = data.frame(round(sigmae,3))
-   rownames(tableS) = "Sigma^2"
+  rownames(tableS) = "Sigma^2"
   colnames(tableS) = c("Est")
   
   if(Arp!=0){
-  tableP  = data.frame(round(phi,3))
-  rownames(tableP) = paste("Phi",1:Arp)
-  colnames(tableP) = c("Est")}
+    tableP  = data.frame(round(phi,3))
+    rownames(tableP) = paste("Phi",1:Arp)
+    colnames(tableP) = c("Est")}
   
   if(Arp==0){
     tableP  = NULL;phi=NULL}
@@ -878,31 +929,32 @@ EMCensArpT<- function(cc,y,x,z,ttc,nj,Arp,initial,cens.type,LL,LU,nu.fixed,iter.
   tableA  = data.frame(round(dd,3))
   rownames(tableA) = paste("Alpha",nnp)
   colnames(tableA) = c("Est")
- 
+  
   
   end.time <- Sys.time()
   time.taken <- end.time - start.time
- 
-    
-    obj.out <- list(beta1 = beta1, sigmae= sigmae, phi=phi, dd = dd,nu=nu, loglik=loglik,
-                    AIC=AICc, BIC=BICc, iter = count,
-                    ubi = ubi, ubbi = ubbi, uybi = uybi, uyi = uyi, uyyi = uyyi,ui=ui, MI=Infbetas,residual=res,
-                    Prev= NULL, time=time.taken, SE=SE,tableB=tableB,tableS=tableS,tableP=tableP,
-                    tableA=tableA,yest=yfit, yog = yhatorg)
-    
-    if  (count == iter.max)
-    {
-      setTkProgressBar(pb, iter.max, label=paste("MaxIter reached ",count,"/",iter.max,"    -    100 % done",sep = ""))
-      close(pb)
-    }
-    else
-    {
-      setTkProgressBar(pb, iter.max, label=paste("Convergence at Iter ",count,"/",iter.max,"    -    100 % done",sep = ""))
-      close(pb)
-    }
-    
+  
+  
+  obj.out <- list(beta1 = beta1, sigmae= sigmae, phi=phi, dd = dd,nu=nu, loglik=loglik,
+                  AIC=AICc, BIC=BICc, iter = count,
+                  ubi = ubi, ubbi = ubbi, uybi = uybi, uyi = uyi, uyyi = uyyi,ui=ui, MI=Infbetas,residual=res,
+                  Prev= NULL, time=time.taken, SE=SE,tableB=tableB,tableS=tableS,tableP=tableP,
+                  tableA=tableA,yfit=yfit, yorg = yhatorg)
+  
+  if  (count == iter.max)
+  {
+    setTkProgressBar(pb, iter.max, label=paste("MaxIter reached ",count,"/",iter.max,"    -    100 % done",sep = ""))
+    close(pb)
+  }
+  else
+  {
+    setTkProgressBar(pb, iter.max, label=paste("Convergence at Iter ",count,"/",iter.max,"    -    100 % done",sep = ""))
+    close(pb)
+  }
+  
   class(obj.out) <- "ARpTLMEC"
   
   return(obj.out)
   
 }
+
